@@ -26,12 +26,28 @@ setAs("matrix", "Matrix", function(from) Matrix(from))
 ## 	  array(from@x, dim = d, dimnames = dimnames(from))
 ##       })
 
-## should propagate to all subclasses:
-setMethod("as.matrix", signature(x = "Matrix"), function(x) as(x, "matrix"))
-## for 'Matrix' objects, as.array() should be equivalent:
-setMethod("as.array",  signature(x = "Matrix"), function(x) as(x, "matrix"))
+.asmatrix <- function(x) as(x, "matrix") # not better; just for those hating typing
+
 ## Such that also base functions dispatch properly on our classes:
-as.array.Matrix <- as.matrix.Matrix <- function(x, ...) as(x, "matrix")
+if(.Matrix.avoiding.as.matrix) {
+    as.matrix.Matrix <- function(x, ...) {
+        if(nonTRUEoption("Matrix.quiet.as.matrix") && nonTRUEoption("Matrix.quiet"))
+            warning("as.matrix(<Matrix>) is deprecated (to become a no-op in the future).
+Use  as(x, \"matrix\")  or .asmatrix(x) instead.")
+        as(x, "matrix")
+    }
+    as.array.Matrix <- function(x, ...) {
+        warning("as.array(<Matrix>) is deprecated. Use  as(x, \"matrix\")  or .asmatrix(x) instead.")
+        as(x, "matrix")
+    }
+} else { ## regularly -- documented since 2005 that this works
+    as.array.Matrix <- as.matrix.Matrix <- function(x, ...) as(x, "matrix")
+}
+
+## should propagate to all subclasses:
+setMethod("as.matrix", signature(x = "Matrix"), function(x) as.matrix.Matrix(x))
+## for 'Matrix' objects, as.array() should be equivalent:
+setMethod("as.array",  signature(x = "Matrix"), function(x)  as.array.Matrix(x))
 
 ## head and tail apply to all Matrix objects for which subscripting is allowed:
 setMethod("head", signature(x = "Matrix"), utils::head.matrix)
@@ -47,6 +63,15 @@ setMethod("as.vector", "Matrix",
 ## S3 dispatch works for base::as.vector(), but S4 dispatch does not
 as.vector.Matrix <- function(x, mode) as.vector(as(x, "matrix"), mode)
 
+if(FALSE) { ## still does not work for c(1, Matrix(2))
+## For the same reason (and just in case) also do both S3 and S4 here:
+c.Matrix <- function(...) unlist(lapply(list(...), as.vector))
+## NB: Must use   signature  '(x, ..., recursive = FALSE)' :
+setMethod("c", "Matrix", function(x, ..., recursive) c.Matrix(x, ...))
+## The above is not sufficient for  c(NA, 3:2, <Matrix>, <matrix>)
+setMethod("c", "numMatrixLike", function(x, ..., recursive) c.Matrix(x, ...))
+}# not yet
+
 setAs("Matrix", "vector",  function(from) as.vector (as(from, "matrix")))
 setAs("Matrix", "numeric", function(from) as.numeric(as(from, "matrix")))
 setAs("Matrix", "logical", function(from) as.logical(as(from, "matrix")))
@@ -364,6 +389,9 @@ setMethod("diag<-", signature(x = "Matrix"),
 setMethod("t", signature(x = "Matrix"),
 	  function(x) .bail.out.1(.Generic, class(x)))
 
+## NB: "sparseMatrix" works via "sparseVector"
+setMethod("rep", "Matrix", function(x, ...) rep(as(x, "matrix"), ...))
+
 setMethod("norm", signature(x = "Matrix", type = "character"),
 	  function(x, type, ...) .bail.out.1("norm", class(x)))
 setMethod("rcond", signature(x = "Matrix", norm = "character"),
